import 'dart:convert';

import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:location/location.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_location/src/models/address.dart';
import 'package:http/http.dart' as http;

abstract class LocationRepository {
  Future<AppResponse<LatLng>> getCurrentLocation();
  Future<AppResponse<List<Address>>> searchLocations(String query);
  Future<AppResponse<Address>> getLocationDetails(String placeId);
  Future<AppResponse<Address>> getLocationDetailsByPosition(LatLng position);
  Future<AppResponse<String>> getAddressDetails(LatLng position);
}

class LocationGoogleRepository extends LocationRepository {
  final String apiKey;
  final String geoLocationUrl =
      "https://maps.googleapis.com/maps/api/place/autocomplete/json?";

  LocationGoogleRepository(this.apiKey);

  @override
  Future<AppResponse<LatLng>> getCurrentLocation() async {
    try {
      Location location = Location();
      PermissionStatus permissionStatus = await location.requestPermission();
      if (permissionStatus != PermissionStatus.granted) {
        return AppResponse.named(error: "Permission not given");
      }
      var position = await location.getLocation();
      return AppResponse.named(
          data: LatLng(position.latitude, position.longitude));
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<Address>> getLocationDetails(String placeId) async {
    String url =
        "https://maps.googleapis.com/maps/api/place/details/json?place_id=$placeId&key=$apiKey";
    http.Response response = await http.get(url);
    var json = jsonDecode(response.body);
    return _parseJson(json["result"]);
  }

  @override
  Future<AppResponse<String>> getAddressDetails(LatLng position) async {
    try {
      String apiUrl =
          "https://maps.googleapis.com/maps/api/geocode/json?latlng=${position.latitude}, ${position.longitude}&key=$apiKey";
      http.Response response = await http.get(apiUrl);
      var json = jsonDecode(response.body);
      var results = json['results'];
      if (results != null && results is List && results.length > 0) {
        var address = results[0]["formatted_address"];
        if (address != null) {
          return AppResponse.named(data: address);
        }
      }
      return AppResponse.named(data: "Uknown Place");
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<Address>> getLocationDetailsByPosition(
      LatLng position) async {
    try {
      String apiUrl =
          "https://maps.googleapis.com/maps/api/geocode/json?latlng=${position.latitude}, ${position.longitude}&key=$apiKey";
      http.Response response = await http.get(apiUrl);
      var json = jsonDecode(response.body);
      var results = json['results'];
      if (results != null && results is List && results.length > 0) {
        return _parseJson(results[0]);
      }
      return null;
    } catch (e) {
      print("Exception $e");
      return null;
    }
  }

  @override
  Future<AppResponse<List<Address>>> searchLocations(String query) async {
    try {
      String url = geoLocationUrl +
          "key=$apiKey" +
          "&" +
          "input=$query" +
          "&sensor=false";
      http.Response response = await http.get(url);
      var json = jsonDecode(response.body);
      if (json["predictions"] != null) {
        var list = List<Address>.from(List.from(json['predictions'])
            .map((item) => Address.named(
                area: item["description"], placeId: item["place_id"]))
            .toList());
        return AppResponse.named(data: list);
      }
      return AppResponse.named(error: "Not Found");
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  AppResponse<Address> _parseJson(result) {
    try {
      List<String> mapTypes = [
        "locality",
        "country",
        "postalCode",
        "administrative",
        "administrative_area_level_1"
      ];
      Map<String, dynamic> addressMap = {};
      var address = result["formatted_address"];
      var addressComponents = result["address_components"];
      addressComponents.forEach((val) {
        if (val["types"] != null) {
          List<String> types = List<String>.from(val["types"]);
          for (String type in types) {
            if (types != null && mapTypes.contains(type)) {
              addressMap[type] = val["short_name"];
              break;
            }
          }
        }
      });
      var locationMap = result["geometry"]["location"];
      LatLng position = LatLng(locationMap["lat"], locationMap["lng"]);
      Address locationRequest = Address.named(
          fullAddress: address,
          latitude: position.latitude,
          longitude: position.longitude,
          locality: addressMap['locality'],
          country: addressMap["administrative"] ??
              addressMap["administrative_area_level_1"],
          pinCode: addressMap["postalCode"]);

      return AppResponse.named(data: locationRequest);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }
}
